package arquivos;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.BindException;
import java.net.ServerSocket;
import java.net.Socket;

/**
 * Esta classe implementa um servidor simples que fica aguardando conexões de
 * clientes. Este servidor, quando conectado, envia uma lista de arquivos locais
 * para o cliente, aguarda a seleção de um nome de arquivo e envia o conteúdo
 * deste arquivo para o cliente.
 */
public class ServidorDeArquivos {

    // Método que permite a execução da classe.
    public static void main(String[] args) {
        try {
            // Criamos a instância de ServerSocket que responderá por solicitações
            // à porta 2048.
            ServerSocket servidor = new ServerSocket(2048);
            // O servidor aguarda "para sempre" as conexões.
            while (true) {
                // Quando uma conexão é feita,
                Socket conexão = servidor.accept();
                // ... o servidor a processa.
                processaConexão(conexão);
            }
        } // Pode ser que a porta já esteja em uso !
        catch (BindException e) {
            System.out.println("Porta já em uso.");
        } // Pode ser que tenhamos um erro qualquer de entrada ou saída.
        catch (IOException e) {
            System.out.println("Erro de entrada ou saída.");
        }
    }

    // Este método atende a uma conexão feita a este servidor.
    private static void processaConexão(Socket conexão) {
        try {
            // Para receber os dados do cliente, o servidor usa uma instância de
            // BufferedReader, pois assumimos que o cliente somente enviará strings para o servidor.
            BufferedReader entrada =
                    new BufferedReader(new InputStreamReader(conexão.getInputStream()));
            // Criamos uma stream para enviar textos e dados para o cliente.
            DataOutputStream saída =
                    new DataOutputStream(conexão.getOutputStream());
            // Mandamos uma mensagem de boas vindas.
            saída.writeUTF("Bem-vindo ao servidor de arquivos locais.");
            saída.writeUTF("=========================================");
            // Enviamos ao cliente a lista de arquivos locais.
            File diretório = new File(".");
            String[] arquivos = diretório.list(); 
            for (int a = 0; a < arquivos.length; a++) {
                saída.writeUTF(arquivos[a]);
            }
            // Aguardamos a seleção do usuário.
            saída.writeUTF("-----------------------------------------");
            saída.writeUTF("Selecione um dos arquivos acima.");
            // Informamos ao cliente que terminamos de mandar texto.
            saída.writeUTF("#####");
            // Garantimos que as mensagens foram enviadas ao cliente.
            saída.flush();
            // Lemos o nome de arquivo selecionado pelo cliente.
            String nomeSelecionado = entrada.readLine();
            // Criamos uma representação do arquivo.
            File selecionado = new File(nomeSelecionado);
            // Enviamos uma mensagem esclarecedora para o cliente.
            saída.writeUTF("Enviando arquivo " + nomeSelecionado + " ("
                    + selecionado.length() + " bytes)");
            saída.flush();
            // Abrimos o arquivo localmente.
            DataInputStream entradaLocal =
                    new DataInputStream(new FileInputStream(selecionado));
            // Lemos todos os bytes do arquivo local, enviando-os para o cliente.
            // Para maior eficiência, vamos ler e enviar os dados em blocos de 25 bytes.
            byte[] arrayDeBytes = new byte[25];
            while (true) {
                // Tentamos ler até 25 bytes do arquivo de entrada.
                int resultado = entradaLocal.read(arrayDeBytes, 0, 25);
                if (resultado == -1) {
                    break;
                }
                // Escrevemos somente os bytes lidos.
                saída.write(arrayDeBytes, 0, resultado);
            }
            // Ao terminar de ler o arquivo local, o fechamos.
            entradaLocal.close();
            // Ao terminar de atender a requisição, fechamos a stream de saída.
            saída.close();
            // Fechamos também a conexão.
            conexão.close();
        } // Se houve algum erro de entrada ou saída...
        catch (IOException e) {
            System.out.println("Erro atendendo a uma conexão !");
            e.printStackTrace();
        }
    }
}
